@@ -1,6 +1,6 @@
 /*
     ccid.c: CCID common code
-    Copyright (C) 2003-2005   Ludovic Rousseau
+    Copyright (C) 2003-2010   Ludovic Rousseau
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -18,7 +18,7 @@
 */
 
 /*
- * $Id: ccid.c 4610 2009-12-13 17:44:01Z rousseau $
+ * $Id: ccid.c 6274 2012-04-13 13:45:16Z rousseau $
  */
 
 #include <stdio.h>
@@ -33,7 +33,6 @@
 #include "defs.h"
 #include "ccid_ifdhandler.h"
 #include "commands.h"
-#include "ccid_usb.h"
 
 #ifdef __APPLE__
 #include <CoreFoundation/CoreFoundation.h>
@@ -63,25 +62,22 @@ int ccid_open_hack_pre(unsigned int reader_index)
 		case CL1356D:
 			/* the firmware needs some time to initialize */
 			(void)sleep(1);
-			ccid_descriptor->readTimeout = 60; /* 60 seconds */
+			ccid_descriptor->readTimeout = 60*1000; /* 60 seconds */
 			break;
 	}
 
 	/* CCID */
-	if (0 == ccid_descriptor->bInterfaceProtocol)
+	if ((PROTOCOL_CCID == ccid_descriptor->bInterfaceProtocol)
+		&& (3 == ccid_descriptor -> bNumEndpoints))
 	{
 #ifndef TWIN_SERIAL
-		/* just wait for 10ms in case a notification is in the pipe */
-#ifndef __APPLE__
-		/* the InterruptRead does NOT timeout (on Mac OS X)
-		 * so we do not do that on Mac OS X */
-		(void)InterruptRead(reader_index, 10);
-#endif
+		/* just wait for 100ms in case a notification is in the pipe */
+		(void)InterruptRead(reader_index, 100);
 #endif
 	}
 
 	/* ICCD type A */
-	if (ICCD_A == ccid_descriptor->bInterfaceProtocol)
+	if (PROTOCOL_ICCD_A == ccid_descriptor->bInterfaceProtocol)
 	{
 		unsigned char tmp[MAX_ATR_SIZE];
 		unsigned int n = sizeof(tmp);
@@ -93,7 +89,7 @@ int ccid_open_hack_pre(unsigned int reader_index)
 	}
 
 	/* ICCD type B */
-	if (ICCD_B == ccid_descriptor->bInterfaceProtocol)
+	if (PROTOCOL_ICCD_B == ccid_descriptor->bInterfaceProtocol)
 	{
 		unsigned char tmp[MAX_ATR_SIZE];
 		unsigned int n = sizeof(tmp);
@@ -115,6 +111,102 @@ int ccid_open_hack_pre(unsigned int reader_index)
 	return 0;
 } /* ccid_open_hack_pre */
 
+#ifndef NO_LOG
+/*****************************************************************************
+ *
+ *					dump_gemalto_firmware_features
+ *
+ ****************************************************************************/
+static void dump_gemalto_firmware_features(struct GEMALTO_FIRMWARE_FEATURES *gff)
+{
+	DEBUG_INFO2("Dumping Gemalto firmware features (%zd bytes):",
+		sizeof(struct GEMALTO_FIRMWARE_FEATURES));
+
+#define YESNO(x) (x) ? "yes" : "no"
+
+	DEBUG_INFO2(" bLogicalLCDLineNumber: %d", gff->bLogicalLCDLineNumber);
+	DEBUG_INFO2(" bLogicalLCDRowNumber: %d", gff->bLogicalLCDRowNumber);
+	DEBUG_INFO2(" bLcdInfo: 0x%02X", gff->bLcdInfo);
+	DEBUG_INFO2(" bEntryValidationCondition: 0x%02X",
+		gff->bEntryValidationCondition);
+
+	DEBUG_INFO(" Reader supports PC/SCv2 features:");
+	DEBUG_INFO2("  VerifyPinStart: %s", YESNO(gff->VerifyPinStart));
+	DEBUG_INFO2("  VerifyPinFinish: %s", YESNO(gff->VerifyPinFinish));
+	DEBUG_INFO2("  ModifyPinStart: %s", YESNO(gff->ModifyPinStart));
+	DEBUG_INFO2("  ModifyPinFinish: %s", YESNO(gff->ModifyPinFinish));
+	DEBUG_INFO2("  GetKeyPressed: %s", YESNO(gff->GetKeyPressed));
+	DEBUG_INFO2("  VerifyPinDirect: %s", YESNO(gff->VerifyPinDirect));
+	DEBUG_INFO2("  ModifyPinDirect: %s", YESNO(gff->ModifyPinDirect));
+	DEBUG_INFO2("  Abort: %s", YESNO(gff->Abort));
+	DEBUG_INFO2("  GetKey: %s", YESNO(gff->GetKey));
+	DEBUG_INFO2("  WriteDisplay: %s", YESNO(gff->WriteDisplay));
+	DEBUG_INFO2("  SetSpeMessage: %s", YESNO(gff->SetSpeMessage));
+	DEBUG_INFO2("  bTimeOut2: %s", YESNO(gff->bTimeOut2));
+	DEBUG_INFO2("  bPPDUSupportOverXferBlock: %s",
+		YESNO(gff->bPPDUSupportOverXferBlock));
+	DEBUG_INFO2("  bPPDUSupportOverEscape: %s",
+		YESNO(gff->bPPDUSupportOverEscape));
+
+	DEBUG_INFO2(" bListSupportedLanguages: %s",
+		YESNO(gff->bListSupportedLanguages));
+	DEBUG_INFO2(" bNumberMessageFix: %s", YESNO(gff->bNumberMessageFix));
+
+	DEBUG_INFO2(" VersionNumber: 0x%02X", gff->VersionNumber);
+	DEBUG_INFO2(" MinimumPINSize: %d", gff->MinimumPINSize);
+	DEBUG_INFO2(" MaximumPINSize: %d", gff->MaximumPINSize);
+	DEBUG_INFO2(" Firewall: %s", YESNO(gff->Firewall));
+	if (gff->Firewall && gff->FirewalledCommand_SW1
+		&& gff->FirewalledCommand_SW2)
+	{
+		DEBUG_INFO2("  FirewalledCommand_SW1: 0x%02X",
+			gff->FirewalledCommand_SW1);
+		DEBUG_INFO2("  FirewalledCommand_SW2: 0x%02X",
+			gff->FirewalledCommand_SW2);
+	}
+
+} /* dump_gemalto_firmware_features */
+#endif
+
+/*****************************************************************************
+ *
+ *					set_gemalto_firmware_features
+ *
+ ****************************************************************************/
+static void set_gemalto_firmware_features(unsigned int reader_index)
+{
+	_ccid_descriptor *ccid_descriptor = get_ccid_descriptor(reader_index);
+	struct GEMALTO_FIRMWARE_FEATURES *gf_features;
+
+	gf_features = malloc(sizeof(struct GEMALTO_FIRMWARE_FEATURES));
+	if (gf_features)
+	{
+		unsigned char cmd[] = { 0x6A }; /* GET_FIRMWARE_FEATURES command id */
+		unsigned int len_features = sizeof *gf_features;
+		RESPONSECODE ret;
+
+		ret = CmdEscape(reader_index, cmd, sizeof cmd,
+			(unsigned char*)gf_features, &len_features);
+		if ((IFD_SUCCESS == ret) &&
+		    (len_features == sizeof *gf_features))
+		{
+			/* Command is supported if it succeeds at CCID level */
+			/* and returned size matches our expectation */
+			ccid_descriptor->gemalto_firmware_features = gf_features;
+#ifndef NO_LOG
+			dump_gemalto_firmware_features(gf_features);
+#endif
+		}
+		else
+		{
+			/* Command is not supported, let's free allocated memory */
+			free(gf_features);
+			DEBUG_INFO3("GET_FIRMWARE_FEATURES failed: " DWORD_D ", len=%d",
+				ret, len_features);
+		}
+	}
+} /* set_gemalto_firmware_features */
+
 /*****************************************************************************
  *
  *					ccid_open_hack_post
@@ -123,15 +215,16 @@ int ccid_open_hack_pre(unsigned int reader_index)
 int ccid_open_hack_post(unsigned int reader_index)
 {
 	_ccid_descriptor *ccid_descriptor = get_ccid_descriptor(reader_index);
+	RESPONSECODE return_value = IFD_SUCCESS;
 
 	switch (ccid_descriptor->readerID)
 	{
 		case GEMPCKEY:
 		case GEMPCTWIN:
-			/* Reader announces TPDU but can do APDU */
+			/* Reader announces TPDU but can do APDU (EMV in fact) */
 			if (DriverOptions & DRIVER_OPTION_GEMPC_TWIN_KEY_APDU)
 			{
-				unsigned char cmd[] = { 0xA0, 0x02 };
+				unsigned char cmd[] = { 0x1F, 0x02 };
 				unsigned char res[10];
 				unsigned int length_res = sizeof(res);
 
@@ -327,10 +420,19 @@ int ccid_open_hack_post(unsigned int reader_index)
 				else
 				{
 					DEBUG_COMM("Failed to load l10n strings");
+					return_value = IFD_COMMUNICATION_ERROR;
 				}
 			}
 			break;
 
+		case HPSMARTCARDKEYBOARD:
+		case HP_CCIDSMARTCARDKEYBOARD:
+			/* the Secure Pin Entry is bogus so disable it
+			 * http://martinpaljak.net/2011/03/19/insecure-hp-usb-smart-card-keyboard/
+			 */
+			ccid_descriptor->bPINSupport = 0;
+			break;
+
 #if 0
 		/* SCM SCR331-DI contactless */
 		case SCR331DI:
@@ -371,7 +473,11 @@ int ccid_open_hack_post(unsigned int reader_index)
 #endif
 	}
 
-	return 0;
+	/* Gemalto readers may report additional information */
+	if (GET_VENDOR(ccid_descriptor->readerID) == VENDOR_GEMALTO)
+		set_gemalto_firmware_features(reader_index);
+
+	return return_value;
 } /* ccid_open_hack_post */
 
 /*****************************************************************************
@@ -488,15 +594,13 @@ void ccid_error(int error, const char *file, int line, const char *function)
 
 		default:
 			if ((error >= 1) && (error <= 127))
-			{
 				(void)snprintf(var_text, sizeof(var_text), "error on byte %d",
 					error);
-				text = var_text;
-			}
 			else
 				(void)snprintf(var_text, sizeof(var_text),
 					"Unknown CCID error: 0x%02X", error);
-				text = var_text;
+
+			text = var_text;
 			break;
 	}
 	log_msg(PCSC_LOG_ERROR, "%s:%d:%s %s", file, line, function, text);
